package edu.toronto.cs.ctl.parser;

import java_cup.runtime.*;
import java.util.*;
import java.io.*;
import edu.toronto.cs.binarytree.*;
import edu.toronto.cs.util.*;

parser code {:

  public static void main (String args[])
    {
      CTLParser parser = new CTLParser ();
      parser.setScanner (new Yylex(System.in));
      parser.parseToBTNode ();
    }

  public CTLParser (InputStream in)
    {
      super(new Yylex(in));
    }

  /**
   ** Overrides the default parse by converting the output into a
   ** binary tree.
   **/
  public BTNode parseToBTNode ()
    {
      try
	{
	  return (BTNode) parse ().value; // it is now SUPER parse!
	}
      catch (Exception ex)
	{ // get rid of that nasty exception
	  throw new NestedRuntimeException (ex);
	}
    }

:}

/* all of the terminal grammar symbols (tokens recognized
by the lexical analyzer) 
note: all binary operators associate from left to right
ops are listed from lowest to highest priority */

//terminal EOF;
terminal SEMI, LP, RP, LB, RB, LCB, RCB, DOT, COMMA, QNMRK;
terminal NOT, EQUAL, NOTEQUAL, OVER, UNDER, AND, OR, BWIMPLIES, FWIMPLIES, IFF;
terminal AX, EX, AF, EF, AG, EG, A, E, UNTIL, WEAK;
terminal String ALPHANUM, NUMBER;

non terminal BTNode CTLexpr;
non terminal BTNode expr, id, num;
non terminal LinkedList idlist;

precedence left SEMI, LP, RP, LB, RB, RCB, LCB, QNMRK;
precedence left ALPHANUM, NUMBER;
precedence right FWIMPLIES, BWIMPLIES;
precedence left IFF;
precedence left OR;
precedence left AND;
precedence left NOT;
precedence left EX, AX, EF, AF, EG, AG, E, A, UNTIL, WEAK;
precedence left OVER, UNDER;
precedence left EQUAL, NOTEQUAL;

CTLexpr	::= expr:ctl SEMI
	{: RESULT = ctl; :}
	| expr:ctl
	{: RESULT = ctl; :}
	 ;

expr	::= id:name
	{: RESULT = name; :}
	| num:n
	{: RESULT = n; :}
	| NOT expr:e
	{: RESULT = new BTNode("!", null, e); :}
	| LP expr:e RP
	{: RESULT = e; :}
	| QNMRK id:ph LCB idlist:props RCB
	{:
	  RESULT = new BTNode ("?" + ph.getData (), null,
			       new BTNode (props, null, null));
	  :}
	| expr:e1 OVER expr:e2
	{: RESULT = new BTNode(">=", e1, e2); :}
	| expr:e1 UNDER expr:e2
	{: RESULT = new BTNode("<=", e1, e2); :}
	| expr:e1 EQUAL expr:e2
	{: RESULT = new BTNode("=", e1, e2); :}
	| expr:e1 NOTEQUAL expr:e2
	{: RESULT = new BTNode("!=", e1, e2); :}
	| expr:e1 AND expr:e2
	{: RESULT = new BTNode("/\\", e1, e2); :}
	| expr:e1 OR expr:e2
	{: RESULT = new BTNode("\\/", e1, e2); :}
	| expr:e1 FWIMPLIES expr:e2
	{: RESULT = new BTNode("->", e1, e2); :}
	| expr:e1 BWIMPLIES expr:e2
	{: RESULT = new BTNode("<-", e1, e2); :}
	| expr:e1 IFF expr:e2
	{: RESULT = new BTNode("<->", e1, e2); :}
	| EX expr:e
	{: RESULT = new BTNode("EX", null, e); :}
	| AX expr:e
	{: RESULT = new BTNode("AX", null, e); :}
	| EF expr:e
	{: RESULT = new BTNode("EF", null, e); :}
	| AF expr:e
	{: RESULT = new BTNode("AF", null, e); :}
	| EG expr:e
	{: RESULT = new BTNode("EG", null, e); :}
	| AG expr:e
	{: RESULT = new BTNode("AG", null, e); :}
	| A LB expr:e1 UNTIL expr:e2 RB
	{: RESULT = new BTNode("AU", e1, e2); :}
	| E LB expr:e1 UNTIL expr:e2 RB
	{: RESULT = new BTNode("EU", e1, e2); :}
        | A LB expr:e1 WEAK expr:e2 RB
        {: RESULT = new BTNode ("AW", e1, e2); :}
        | E LB expr:e1 WEAK expr:e2 RB
        {: RESULT = new BTNode ("EW", e1, e2); :}
	;

idlist  ::= id:name
	{:
	  LinkedList temp = new LinkedList ();
	  temp.addFirst (name);
	  RESULT = temp;
        :}
        | idlist:l COMMA id:name
        {:
	  l.addFirst (name);
	  RESULT = l; 
        :}
        ;


id	::= ALPHANUM:a
	{: RESULT = new BTNode(a); :}
	| id:name DOT ALPHANUM:a
	{:
	  name.setData (name.getData() + "." + a );
	  RESULT = name;
	:}
	;

num	::= NUMBER:n
	{: RESULT = new BTNode(n.toString ()); :}
	/* for now numbers are still treated as strings */
	;
