package edu.toronto.cs.experiments;

import java.io.*;
import java.util.*;


/**
 * <code>ExperimentOutput</code> encapsulates experiment output
 *
 * @author <a href="mailto:arie@cs.toronto.edu">Arie Gurfinkel</a>
 * @version 1.0
 */
public class ExperimentOutput
{
  
  public static String SUFFIX = "dat";
  /**
   * <code>name</code> name of the experiment
   *
   */
  String name;
  String dirName;
  
  // -- output files
  Map outputs;

  public ExperimentOutput (String _dirName, String _name)
  {
    dirName = _dirName;
    name = _name;
    outputs = new HashMap ();
  }

  
  public void addSample (String suffix, int data)
  {
    PrintWriter out = getWriter (suffix);
    out.print (data);
    out.print ("\t");
  }
  
  public void flush () 
  {
    for (Iterator it = outputs.values ().iterator (); it.hasNext ();)
      {
	PrintWriter out = (PrintWriter)it.next ();
	out.flush ();
      }
  }
  
  public void close ()
  {
    for (Iterator it = outputs.values ().iterator (); it.hasNext ();)
      {
	PrintWriter out = (PrintWriter)it.next ();
	out.close ();
      }
  }
  

  public PrintWriter getWriter (String suffix)
  {
    try {
      PrintWriter out = (PrintWriter)outputs.get (suffix);

      if (out == null)
	{
	  out = new PrintWriter 
	    (new FileWriter (dirName + "/" + 
			     name + "_" + suffix + "." + SUFFIX));
	  outputs.put (suffix, out);
	}
      
      return out;
    }
    catch (IOException ex) {
      ex.printStackTrace ();
    }
    return null;
  }
}
