// first set of experiments

package edu.toronto.cs.experiments;

import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.util.Chaos;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.io.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import org.apache.log4j.*;
import java.util.Collection;
import java.text.NumberFormat;

public class First {

    Algebra fa;
    int nv;
    MvSetFactory mvsf, jmvsf, cubdd;
    Map mvsetFactories;

    Function fn;


    static Category log = 
	Category.getInstance("edu.toronto.cs.experiments.First");

    public First(Algebra logic, int nVars, Function fn_) {

	nv = nVars;
	fa = logic;
	mvsetFactories = new HashMap();

 	mvsf = new MDDMvSet.MDDMvSetFactory(logic, nVars);
 	mvsetFactories.put("MDD", mvsf);
 	jmvsf = new JMvSet.JMvSetFactory((FiniteLattice)logic, nVars);
 	mvsetFactories.put("JIMDD", jmvsf);
	//	cubdd = CUBDDMvSet.newMvSetFactory ((FiniteLattice)logic, nVars);
	//	mvsetFactories.put ("CUBDD", cubdd);
	fn = fn_.clone(logic.size(), nVars);
	//	fn = new RandomFunction(logic.size(), nVars);
    }

    
    public static First getByStateSize(String logic, int stateSpace, Function fn_) {
	Algebra lgc=null;
	lgc = Lattices.getLatticeByName(logic);
	int nvrs = (int)
	    Math.ceil(Math.log( (double)stateSpace)/
		      Math.log((double)lgc.size()));
	log.info("state space:"+stateSpace+", variable size:"+lgc.size()+", nvars:"+nvrs);
	return new First(lgc, nvrs, fn_);
    }


    public First(String logic, int nVars, Function fn_) {
	nv = nVars;
	Algebra lgc = null;
	mvsetFactories = new HashMap();
	lgc = Lattices.getLatticeByName(logic);
	
	fa = lgc;
 	mvsf = new MDDMvSet.MDDMvSetFactory(lgc, nVars);
 	mvsetFactories.put("MDD", mvsf);
 	jmvsf = new JMvSet.JMvSetFactory((FiniteLattice)lgc, nVars);
 	mvsetFactories.put("JIMDD", jmvsf);
	//	cubdd = CUBDDMvSet.newMvSetFactory ((FiniteLattice)lgc, nVars);
	//	mvsetFactories.put ("CUBDD", cubdd);
	
	fn = fn_.clone(lgc.size(), nVars);
    }

  public MvSet addMinterm(MvSet accum, int iVV[], int out) 
  {
    
    return accum.or(accum.getFactory().createPoint(iVV,out));
  }
    
  public MvSet addMintermj(MvSet accum, int iVV[], int out) 
  {
    return accum.or(jmvsf.createPoint(iVV,out));
  }

  public MvSet addMintermCubdd (MvSet accum, int iVV[], int out)
  {
    return accum.or (cubdd.createPoint (iVV, out));
  }
  

  public int[] repSize() 
  {

    int mx = fa.size () - 1;
    int[] res = new int [mvsetFactories.size()];
    MvSet[] m = new MvSet [mvsetFactories.size()];
    String[] names = new String [mvsetFactories.size()];

    Iterator it = mvsetFactories.keySet().iterator();
    int i = 0;
    while (it.hasNext ())
      {
		
	names[i] = (String)it.next();
	m [i] = 
	  ((MvSetFactory)mvsetFactories.get(names[i])).createConstant(mx);

	Iterator innerIt = fn.getAllPoints();
	while (innerIt.hasNext()) {
	  Function.Entry curr =
	    (Function.Entry) innerIt.next();
	  m[i] = addMinterm(m[i], curr.inputs, mx - curr.output);

	}

	res[i] = m[i].size ();
	i++;
      }
	
    return res;
  }

    public void advance() {
	log.info("Advancing.");
	fn.refill();
    }

  public void sample (String title, PrintWriter pw, PrintWriter pw2, long n) 
  {	
    //	pw.print(title+":\t");
    //	pw2.print(title+"(JI):\t");
    for (long i = 0; i < n; i++)
      {
	int[] x = repSize ();
	advance ();
	System.out.print (".");
	pw.print (x[0]);
	pw.print ("\t");
	pw.flush ();
	pw2.print (x[1]);
	pw2.print ("\t");
	pw2.flush ();
      }
    pw.println ("");
    pw.flush ();
    pw2.println ("");
    pw2.flush ();
	
  }

    public static void sparseInputExp() {
	float sparse = (float)0.1;
	for (int i=1; i<=10; i++) {
	    float dis = (float)i/(float)10;
	    Iterator latIt = Lattices.availableLattices().iterator();
	    while (latIt.hasNext()) {
		
		String curLat = (String)latIt.next();
		System.out.println("lattice "+curLat+" dis="+dis);
		First f = getByStateSize(curLat, 1024, new SparseInputBoolOutput(2, 1, sparse, dis));
  		PrintWriter sopw = null, sopwj = null;		    
		
		
		try {
		    sopw = new PrintWriter(new
					   BufferedWriter(new
							  OutputStreamWriter(new
									     FileOutputStream("sib"+curLat+Float.toString(dis)+".dat")
									     )
							  )
					   );
		    sopwj = new PrintWriter(new
				 BufferedWriter(new
						OutputStreamWriter(new
								   FileOutputStream("sib"+curLat+Float.toString(dis)+"j.dat")
								   )
						)
					    );
		} // end try
		catch (Exception e) { e.printStackTrace(); }
		f.sample(curLat+Float.toString(dis), sopw, sopwj, 10);
	    }
	}      
    }

    // use: allLogicsConstantVariables(5, "random", new RandomFunction(2,2));
  public static void allLogicsConstantVariables (int nvars, String kase, 
						 Function funk) 
  {
    Iterator latIt = Lattices.availableLattices ().iterator ();

    while (latIt.hasNext ())
      {
	String curLat = (String)latIt.next ();
	System.out.println ("lattice "+curLat);

	First f = new First (curLat, nvars, funk);
	PrintWriter sopw = gimmeWriter (kase+"_"+curLat+".dat"), 
	  sopwj = gimmeWriter (kase+"_"+curLat+"_j.dat");

	f.sample(curLat + "("+kase+")", sopw, sopwj, 10);
	    

      } // end while	
  }

    public static void allLogicsConstantState(int stateSize, String kase, Function funk) {
	Iterator latIt = Lattices.availableLattices().iterator();
  	while (latIt.hasNext())
  	    {
  		String curLat = (String)latIt.next();
  		log.debug("lattice "+curLat);

		First f = getByStateSize(curLat, stateSize, funk);
  		PrintWriter sopw = gimmeWriter(kase+"_"+curLat+".dat"), 
		    sopwj = gimmeWriter(kase+"_"+curLat+"_j.dat");

  	f.sample(curLat+"("+kase+")", sopw, sopwj, 10);
	    

	    } // end while	
    }

    public static void sparseInputMVOutExp() {
	float sparse = (float)0.1;
	for (int i=1; i<=10; i++) {
	    float dis = (float)i/(float)10;
	    Iterator latIt = Lattices.availableLattices().iterator();
	    while (latIt.hasNext()) {
		
		String curLat = (String)latIt.next();
		System.out.println("lattice "+curLat+" dis="+dis);
		First f = getByStateSize(curLat, 1024, new SparseInputMVOutput(2, 1, sparse, dis));
  		PrintWriter sopw = null, sopwj = null;		    
		
		
		try {
		    sopw = new PrintWriter(new
					   BufferedWriter(new
							  OutputStreamWriter(new
									     FileOutputStream("simv"+curLat+Float.toString(dis)+".dat")
									     )
							  )
					   );
		    sopwj = new PrintWriter(new
				 BufferedWriter(new
						OutputStreamWriter(new
								   FileOutputStream("simv"+curLat+Float.toString(dis)+"j.dat")
								   )
						)
					    );
		} // end try
		catch (Exception e) { e.printStackTrace(); }
		f.sample(curLat+Float.toString(dis), sopw, sopwj, 10);
	    }
	}      
    }

  public static void main(String argv[]) 
  {
    Lattices.initialize();
    //	Category.getDefaultHierarchy().disableDebug();
    Chaos.seed(23);

    // random
    //	allLogicsConstantVariables(2, "randomA", new RandomFunction(2,2));
    //allLogicsConstantState(1024, "randomB", new RandomFunction(2,2));
    System.out.println("Got here");

    String expName = "xtest";
    int numVar = 10;
    
    for (Iterator it = Lattices.latticeNames.iterator ();
	 it.hasNext ();)
      {
	String latticeName = it.next ().toString ();
	System.out.println ("\nDoing lattice: " + latticeName);
	    
	Lattices.setSpecific (latticeName); 
	for (int i = 10; i <= 60; i += 10) {
//  	    allLogicsConstantVariables(numVar, expName + Integer.toString(i),
//  				       new ModelCheckingFunction(Lattices.getLatticeByName(latticeName), 10, 3, 5, i));
	    //  	  allLogicsConstantVariables (numVar, expName + Integer.toString (i), 
	    //				     new StuffFunction(2, 5, 3, 5, i));
	}
      }

    System.exit (0);
    System.out.println ("Filling in the half intervals");
    for (Iterator it = Lattices.latticeNames.iterator ();
	 it.hasNext ();)
      {
	String latticeName = it.next ().toString ();
	System.out.println ("\nDoing lattice: " + latticeName);
	    
	Lattices.setSpecific (latticeName); 
	for (int i=15; i<=60; i+=10) {
	  allLogicsConstantVariables(numVar, expName + Integer.toString(i), 
				     new StuffFunction(2, 5, 3, 5, i));
	}
      }
	
    //	varyVariablesOneLogic("6Coffee", 0.05, 0.5, 0.05, 2, 2, 6);

    // -- ARIE -- 5 variables
    //	sparseRandomExperimentOnly (0.1, 0.5, 0.05, 3, 5);
    // -- Ouch
    System.exit (0);

    sparseRandomExperiment(0.1, 0.5, 0.05, 3, 2, 1024);

    sparseInSkewOut(0.1, 0.8, 0.1,
		    0.0, 1.0, 0.1,
		    1, 1,
		    2, 1024,
		    (float)0.1);
    System.exit(0);

    // sparse input, bool output
    for (int sp = 1; sp<9; sp++) {
      float spf = (float)sp / (float)10;
      allLogicsConstantVariables(2, "sparseboolA"+Float.toString(spf), new SparseInputBoolOutput(2,2,(float)0.1, spf));
      allLogicsConstantState(1024, "sparseboolB"+Float.toString(spf), new SparseInputBoolOutput(2,2,(float)0.1, spf));
    }

    // sparse input, mv output
    for (int sp = 1; sp<9; sp++) {
      float spf = (float)sp / (float)10;
      allLogicsConstantVariables(2, "sparsemvA"+Float.toString(spf), new SparseInputMVOutput(2,2,(float)0.1, spf));
      allLogicsConstantState(1024, "sparsemvB"+Float.toString(spf), new SparseInputMVOutput(2,2,(float)0.1, spf));
    }


  } // end main



    protected static PrintWriter gimmeWriter(String str) {
	PrintWriter ret = null;
  	try {
  	 ret = new PrintWriter(new
  					   BufferedWriter(new
  							 OutputStreamWriter(new
  									    FileOutputStream(str)
  									    )
  							 )
  					   );

  	} // end try
	catch (Exception e) {e.printStackTrace(); }
	return ret;
    }


    public static void sparseRandomExperiment(double lower,
					      double upper,
					      double step,
					      int sigdig,
					      int vars,
					      int stateSpace) {

	// sparse
	NumberFormat nf = NumberFormat.getInstance();
	nf.setMaximumFractionDigits(sigdig);	
	nf.setMinimumFractionDigits(sigdig);

	for (double spf = lower; spf<(upper+step); spf+=step) {
	    allLogicsConstantVariables(vars, "sparseA"+nf.format(spf), new SparseFunction(2,2,(float)spf));
	    allLogicsConstantState(stateSpace, "sparseB"+nf.format(spf), new SparseFunction(2,2,(float)spf));
	}

    }


    public static void varyVariablesOneLogic(String latticeName,
					     double lower,
					     double upper,
					     double step,
					     int sigdig,
					     int lowvars,
					     int highvars) {
	NumberFormat nf = NumberFormat.getInstance();
	nf.setMaximumFractionDigits(sigdig);	
	nf.setMinimumFractionDigits(sigdig);

	
	Lattices.setSpecific (latticeName);
	for (int vars = lowvars; vars<=highvars; vars++) {
	    System.out.println ("\nDoing lattice: " + latticeName+
				" with "+vars+" variables.");
	    for (double spf = lower; spf<(upper+step); spf+=step) {
		allLogicsConstantVariables(vars, 
					   "sparseA"+Integer.toString(vars)+"_"+
					   nf.format(spf), 
					   new SparseFunction(2,2,(float)spf));
		// -- make an explicit call to garbage collection
		System.gc ();
	    }
	}
	
    }
  public static void sparseRandomExperimentOnly (double lower,
						 double upper,
						 double step,
						 int sigdig,
						 int vars)
  {
    
    
	// sparse
	NumberFormat nf = NumberFormat.getInstance();
	nf.setMaximumFractionDigits(sigdig);	
	nf.setMinimumFractionDigits(sigdig);

	for (Iterator it = Lattices.latticeNames.iterator ();
	     it.hasNext ();)
	  {
	    String latticeName = it.next ().toString ();
	    System.out.println ("\nDoing lattice: " + latticeName);
	    
	    Lattices.setSpecific (latticeName);
	    for (double spf = lower; spf<(upper+step); spf+=step) {
	      allLogicsConstantVariables(vars, 
					 "sparseA"+nf.format(spf), 
					 new SparseFunction(2,2,(float)spf));
	      // -- make an explicit call to garbage collection
	      System.gc ();
	    }
	  }
	

    }

		    
    public static void sparseInSkewOut(double splow,
				       double sphigh,
				       double spstep,
				       double sklow,
				       double skhigh,
				       double skstep,
				       int spsigdig,
				       int sksigdig,
				       int vars,
				       int stateSpace,
				       float density) {
	// sparse input, mv (skewed) output
	NumberFormat nf1 = NumberFormat.getInstance(), nf2=NumberFormat.getInstance();
	nf1.setMaximumFractionDigits(spsigdig);	
	nf1.setMinimumFractionDigits(spsigdig);
	nf2.setMaximumFractionDigits(sksigdig);	
	nf2.setMinimumFractionDigits(sksigdig);
	
	for (double sp = splow; sp<(sphigh+spstep); sp+=spstep) {
	    for (double sk = sklow; sk<(skhigh+skstep); sk+=skstep) {
	    float spf = (float)sp;
	    float sku = (float)sk;
	    allLogicsConstantVariables(2, "sparsemvA"+nf1.format(sp)+
				       "skew"+nf2.format(sk), 
				       new 
				       SparseInputMVSkew(2,2,
							   density, (float)sp,
							   (float)sk));
	    allLogicsConstantState(1024, "sparsemvB"+nf1.format(sp)+
				   "skew"+nf2.format(sk), 
				   new 
				   SparseInputMVSkew(2,2,
						     density, (float)sp,
						     (float)sk));
	    }
	}
    }

    
}

