package edu.toronto.cs.gui;

import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.ctl.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.tlq.*;
import edu.toronto.cs.smv.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.keg.*;
import edu.toronto.cs.util.dgraph.*;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

import javax.swing.table.*;

/**
 * LatticeTable.java
 *
 *
 * An extension of JTable. Displays the lattice table.
 *
 * Created: Tue Jun 25 13:17:25 2002
 *
 * @author <a href="mailto:anya@tallinn.cs">Tafliovich Anya</a>
 * @version
 */

public class LatticeTable extends JTable {

  // the table data model
  LatticeTableModel tableModel;

  static final int rowHeight = 20;
  static final int viewportWidth = 300;
  static final int viewportHeight = 100;

  /** Constructor
   ** create a scroll panel, which contains a table
   ** describing the operation 'type' (see FiniteLattice)
   ** on the values of 'lattice'
   **/
  public LatticeTable (FiniteLattice lattice, int type)
  {
    super();

    tableModel = new LatticeTableModel (lattice, type);

    setModel (tableModel);

    // allow single cell selection
    setCellSelectionEnabled (true);

    // aloow single selections only
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);


    // color selected cells 
    setSelectionBackground (Color.yellow);
    
    setRowHeight (rowHeight);
    
    // set the scrollable size
    //  setPreferredScrollableViewportSize (new Dimension (viewportWidth,
    //					       viewportHeight));
    
  }

  public LatticeTable (TableModel m)
  {
    super (m);
  }

  // the first column gets the same cell renderer as
  // the table header; all other columns get the default cell
  // renderer 
  public TableCellRenderer getCellRenderer(int row, int col) {
    if (col == 0) 
      return super.getTableHeader().getDefaultRenderer();
    
    return super.getCellRenderer(row, col);
  }

}// LatticeTable
