package edu.toronto.cs.heuristics;

import java.util.*;

import edu.toronto.cs.ctl.*;
import edu.toronto.cs.algebra.*;

/**
 ** Heuristic analyser for the counter-example generator.
 **/

public class SimpleHeuristicAnalyser implements HeuristicAnalyser
{

  public Set getInterestingAnd (Set ctl)
  {
    // XXX Replaced it with the full code to avoid using negation
    // return negate (getInterestingOr (negate (ctl)));
    
    Set result = new HashSet (ctl);
    
    CTLNode node;
    
    node = (CTLNode)result.iterator ().next ();
    FiniteLattice lattice = ((CTLLatticeNode)node.getRight ()).getLattice ();
    
    BitSet vals = new BitSet (lattice.size ());
    
    // -- collect values
    for (Iterator it = result.iterator (); it.hasNext ();)
      {
	node = (CTLNode)it.next ();
	vals.set (((CTLLatticeNode)node.getRight ()).getLatIdx ());
      }

    // -- reduce
    vals = lattice.getIrredundantMeet (vals);
    
    // -- remove formulas that are redundant
    for (Iterator it = result.iterator (); it.hasNext ();)
      {
	node = (CTLNode)it.next ();
	if (!vals.get (((CTLLatticeNode)node.getRight ()).getLatIdx ()))
	  it.remove ();
      }
    return result;
  }

  public Set getInterestingOr (Set ctl)
  {

    // XXX given a set of CTL formulas returns a  subset that
    // XXX when joined results in the final value
    // XXX for example if 'ctl' consists of 3 formulas a, b, and c
    // XXX such that a and b evaluate to T and c to F, returns a, b only
    
    Set result = new HashSet (ctl);
    CTLNode p;

    // -- get the lattice
    p = (CTLNode) result.iterator ().next ();
    FiniteLattice lat = ((CTLLatticeNode)
			 p.getRight ()).getLattice ();

    // -- create bit set large enough to represent every value in the lattice
    BitSet vals = new BitSet (lat.size ());


    // -- iterate through all of the formulas, and collect 
    // -- the value of each. We assume that 'ctl' is a set 
    // -- of formulas of the type 'ctl_exp = lat_value'
    for (Iterator i = result.iterator (); i.hasNext (); )
      {
	p = (CTLNode) i.next ();
	vals.set (((CTLLatticeNode) p.getRight ()).getLatIdx ());
      }

    // -- find out which values are wanted
    vals = lat.getIrredundantJoin (vals);
    // -- remove the otehr values
    for (Iterator i = result.iterator (); i.hasNext (); )
      {
	p = (CTLNode) i.next ();
	if (!vals.get (((CTLLatticeNode) p.getRight ()).getLatIdx ()))
	  i.remove ();
      }

    return result;
  }


  /**
   * for each formula of the type ctl_exp = lat_val
   * computes !lat_val and 
   * creates ctl_exp = !lat_val formula
   */
  Set negate (Set ctl)
  {
    
    Set result = new HashSet ();

    CTLNode p;
    CTLNode val;

    // obtain the lattice
    p = ((CTLNode) ctl.iterator ().next ());
    FiniteLattice lat = ((CTLLatticeNode)
			 p.getRight ()).getLattice ();

    // -- process each node in 'ctl' set
    for (Iterator i = ctl.iterator (); i.hasNext (); )
      {
	p = (CTLNode) i.next ();
	// -- create a node corresponding to the negation 
	// -- of the current value
	val = CTLFactory.createCTLLatticeNode 
	  ((lat.unOp (((CTLLatticeNode) p.getRight ()).getLatIdx (),
		      FiniteLattice.NEG)), lat);
	
	// -- add new formula to the result set
	result.add (p.getLeft ().eq (val));	
      }
    
    return result;
  }

}
