package edu.toronto.cs.keg;

import java.util.*; // We all love java.util

import edu.toronto.cs.ctl.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.util.dgraph.*;

import edu.toronto.cs.mvset.MvState;

public class KEGStateNode extends DGUniqueNode implements KEGNode
{
  static Logger log = new Logger ("KEGStateNode");
  

  DGraph ctl;
  HashMap allChildren;
  MvState state;

  protected KEGStateNode (MvState _state, DGNodeFactory factory)
    {
      super (_state, factory);
      allChildren = new HashMap ();
      ctl = new DGraph (new KEGCtlNodeFactory ());
      state = _state;

    }

  public void addNewSuccessor (MvState _state)
    {
      KEGStateNode state = 
	((KEGStateNodeFactory)owner).createNode (this, _state);
      add (state); 
    }

  public void addNewSuccessor (MvState _state, int transition)
  {
    addNewSuccessor (_state);
    // XXX Assumes various things, all internal to this class, but
    // still ugly.  not least, this entire method now needs to be
    // synchronised if we use it in a multi-threaded environment...
    KEGStateNode child = (KEGStateNode)allChildren.get (_state);
    KEGStateTransition edge = (KEGStateTransition) getEdge (child);
    edge.setValue (transition);
  }

  public MvState getState ()
    {
      return state;
    }
  

  public DGraph getCTLGraph ()
    {
      return ctl;
    }

  public Object getIdent ()
  {
    return state;
  }
  
  // Factory method for standard edge...
  public void setupEdge (DGNode successor)
  {
    edgeData.put (successor, new KEGStateTransition (this, successor));
  } 
  

  public void setCTLGraph (DGraph _ctl)
    {
      ctl = _ctl;
    }

  public Set getChildren ()
    {
      return (Set)getSuccessors();
    }

  //public KEGStateNodeFactory getFactory ()
  //  {
  //    return (KEGStateNodeFactory)super.getFactory ();
  //  }
  
  public void add (KEGStateNode node, int transition)
    {
      add (node);
      //allChildren.put (((KEGStateNode)node).getState (), node);
      edgeData.put (node, new KEGStateTransition (this, node, transition));
    }

  public boolean add (KEGStateNode node)
  {
    return super.add (node);
    //allChildren.put (((KEGStateNode)node).getState (), node);
  }
  

  public boolean add (DGNode successor)
    {
      return add ((KEGStateNode)successor);
    }

  public boolean checkValid(DGNode node)
    {
      return super.checkValid(node) && (node instanceof KEGStateNode);
    }

  public void accept (KEGraphVisitor visitor, Object stateinfo)
  {
    log.debug ("VISIT KEGState visited by KEGV");
    visitor.visitKStateNode (this, stateinfo);
  }

  //public void accept (KEGraphVisitor visitor, Object stateinfo)
  //{
  //  visitor.visitKStateNode (this, stateinfo);
  //}
  

}









