package edu.toronto.cs.lift;

import edu.toronto.cs.smv.*;
import edu.toronto.cs.mvset.*;

public class BoolButton extends SMVModule
{
  static final int PRESSED = 0;
  static final int REQUEST = 1;

  MvSet reset = null;
  
  public BoolButton (String _name)
  {
    super (_name);
    vars = new String[]{"pressed", "request"};
  }
  

  public void setReset (MvSet v)
  {
    reset = v;
  }
  
  
  public MvSet computeTrans ()
  {
    System.out.println ("Computing trans for: " + getName ());
    return computeRequest ().and (computePressed ());
  }  

  public MvSet computePressed ()
  {
    // -- pressed := {true, false}
    return next (PRESSED, top).or (next (PRESSED, bot));
  }
  
  public MvSet computeRequest ()
  {
    /***
     * request := case
     *             pressed & !reset: true;
     *             request & !reset: true;
     *      	   1               : false;
     *		  esac
     */
    MvSet requestIsRequest = 
      ddManager.createProjection (getVarOrder (REQUEST)).eq
      (ddManager.createProjection (getNextVarOrder (REQUEST)));
    /**
     * request := case
     *             reset : false;
                   pressed: true;
		   1 : request
    */

    return new BoolCaseStatement (mvBot).
      addCase (reset, next (REQUEST, bot)).
      addCase (cur (PRESSED), next (REQUEST)).
      addDefault (requestIsRequest).compute ();
  }

  

  public MvState initializeInitState (MvState initState)
  {
    //initState.setValue (getVarOrder (PRESSED), bot);
    initState.setValue (getVarOrder (PRESSED), 
			lattice.getElementIdxByKey ("M"));
    initState.setValue (getVarOrder (REQUEST), bot);
    return initState;
  }
  
    
}
