package edu.toronto.cs.lift;

import edu.toronto.cs.smv.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.mvset.CUBDDMvSet.*;
import edu.toronto.cs.mvset.MDDMvSet.*;
import edu.toronto.cs.mvset.JMvSet.*;
import edu.toronto.cs.mvset.JBDDMvSet.*;
import edu.toronto.cs.mvset.JADDMvSet.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.cudd.Cudd;

import java.util.Arrays;


public class MainLiftRCombined extends SMVModule
{

  SMVModule liftEFTT;
  SMVModule liftTTEF;
  
  FiniteLattice twoVal;
  FiniteLattice threeVal;
  FiniteLattice fourVal;

  MvSetFactory dstFactory;

  MvState boolInitState;

  public MainLiftRCombined (int floors) 
  {
    super ("main");
    
    try {
      twoVal = LatticeFactory.parse ("play/gxl/boolean.xml");
      fourVal = LatticeFactory.parse ("play/models/phone/4val-logic.xml");
      threeVal = LatticeFactory.parse ("play/gxl/3val-logic.xml");
    } catch (Exception ex) {
      Assert.assert ("Hmm", ex);
    }

    liftEFTT = new MainLiftREFTT ("main", floors);
    //liftTTEF = new MainLiftREFTT ("main", floors);
    liftTTEF = new MainLiftRTTEF ("main", floors);
    //liftEFTT = new MainLiftRTTEF ("main", floors);
    
    setLattice (twoVal);
    //setLattice (fourVal);
  }
  

  public void setMvSetFactory (MvSetFactory factory)
  {
    dstFactory = factory;
    MvSetFactory boolFactory = null;
    
    if (dstFactory instanceof MDDMvSetFactory)
      boolFactory = MDDMvSet.newMvSetFactory (twoVal, ordering.size ());
    else if (dstFactory instanceof JMvSetFactory)
      boolFactory = JMvSet.newMvSetFactory (twoVal, ordering.size ());
    else if (dstFactory instanceof JBDDMvSetFactory)
      boolFactory = JBDDMvSet.newMvSetFactory (twoVal, ordering.size ());
    else if (dstFactory instanceof JADDMvSetFactory)
      boolFactory = JADDMvSet.newMvSetFactory (twoVal, ordering.size ());
    else
      Assert.assert ("Hmm problems!");
//      MvSetFactory boolFactory = 
//        CUBDDMvSet.newMvSetFactory (twoVal, ordering.size ());
    super.setMvSetFactory (boolFactory);
    liftEFTT.setMvSetFactory (boolFactory);
    liftTTEF.setMvSetFactory (boolFactory);
  }
  public void setLattice (FiniteLattice lattice)
  {
    liftEFTT.setLattice (lattice);
    liftTTEF.setLattice (lattice);
  }
  public FiniteLattice getLattice ()
  {
    return fourVal;
    //return threeVal;
  }
  public String[] getVars ()
  {
    // -- this is needed for initialization
    liftTTEF.getVars ();
    return liftEFTT.getVars ();
  }
  public void setVarOrdering (PropOrderIf ordering)
  {
    this.ordering = ordering;    
    System.out.println ("Calling setVarOrdering on lifts");
    liftEFTT.setVarOrdering (ordering);
    liftTTEF.setVarOrdering (ordering);
  }
  
  

  public MvSet computeTrans ()
  {
    //return mergeMvSets (liftEFTT.computeTrans (), liftTTEF.computeTrans ());
//      MvSet trans = liftEFTT.computeTrans ();
//      return mergeMvSets (trans, trans);
    return mergeMvSets (liftEFTT.computeTrans (), liftTTEF.computeTrans ());
  }

  MvSet mergeMvSets (MvSet set1, MvSet set2)
  {
    if (set1 instanceof MDDMvSet)
      return mergeMvSetsMDD (set1, set2);
    else if (set1 instanceof JMvSet)
      return mergeMvSetsJoin (set1, set2);
    else if (set1 instanceof JADDMvSet)
      return mergeMvSetsAdd (set1, set2);
    else if (set1 instanceof JBDDMvSet)
      return mergeMvSetsBdd (set1, set2);
    return null;
  }

  MvSet mergeMvSetsAdd (MvSet _set1, MvSet _set2)
  {
    System.out.println ("Merging mvsets -- ADD");
    JADDMvSet set1 = (JADDMvSet)_set1;
    JADDMvSet set2 = (JADDMvSet)_set2;
    
    MDDManager srcManager = ((JADDMvSetFactory)ddManager).getMddManager ();
    MDDManager dstManager = ((JADDMvSetFactory)dstFactory).getMddManager ();

    MDDNode node1 = set1.getAdd ();
    MDDNode node2 = set2.getAdd ();
    
    int[] logicMap = new int [2];
    
    logicMap [twoVal.getElementIdxByKey ("T")]
      = fourVal.getElementIdxByKey ("TT");
    logicMap [twoVal.getElementIdxByKey ("F")]
      = fourVal.getElementIdxByKey ("FF");
    
    
    System.out.println ("Transfering node1");
    node1 = transferAdd (srcManager, dstManager, node1, logicMap);
    System.out.println ("Transfering node2");
    node2 = transferAdd (srcManager, dstManager, node2, logicMap);
//      node1 = srcManager.transfer (dstManager, node1, logicMap, 2);
//      node2 = srcManager.transfer (dstManager, node2, logicMap, 2);
    
    
    System.out.println ("Composing");
    MDDNode compose = 
      dstManager.Apply (node1, node2, FiniteLattice.COMPOSE);
    
    MvSet result = ((JADDMvSetFactory)dstFactory).createMvSet (compose);
    
    liftEFTT = null;
    liftTTEF = null;
    srcManager = null;
    ddManager = null;
    
    System.out.println ("Garbage collecting");
    System.gc ();
    return result;
  }
  
  MDDNode transferAdd (MDDManager srcManager, MDDManager dstManager, 
		      MDDNode node, int[] logicMap)
  {
    if (node.isConstant ())
      return dstManager.getConstant (logicMap [node.getValue ()]);
    
    int nodeIndex = node.getVarIndex () << 1;

    MDDNode[] children = new MDDNode [dstManager.getAlgebra ().size ()];
    Arrays.fill (children, dstManager.getZero ());
    
    for (int i = 0; i < node.howManyChildren (); i++)
      {
	if (logicMap [i] == Algebra.NO_VALUE) continue;
	
	MDDNode extraNode = 
	  dstManager.getVarEquals (nodeIndex + 1, logicMap [i]);
	
	children [logicMap [i]] =
	  dstManager.Apply (extraNode, 
			    transferAdd (srcManager, dstManager, 
					 node.getChild (i), logicMap),
			    FiniteLattice.MEET);
      }
    
    return dstManager.makeUnique (nodeIndex, children);
    
  }
  
    
  MvSet mergeMvSetsBdd (MvSet _set1, MvSet _set2)
  {
    System.out.println ("Merging mvsets -- BDD");
    JBDDMvSet set1 = (JBDDMvSet)_set1;
    JBDDMvSet set2 = (JBDDMvSet)_set2;
    

    MDDManager srcManager = ((JBDDMvSetFactory)ddManager).getMddManager ();
    MDDManager dstManager = ((JBDDMvSetFactory)dstFactory).getMddManager ();

    MDDNode[] nodes = new MDDNode [2];
    
    nodes [0] = set1.getMddNodes () [0];
    nodes [1] = set2.getMddNodes () [0];

    int[] logicMap = new int [] {0, 1};
    System.out.println ("Transfering the first");
    nodes [0] = srcManager.transfer (dstManager, nodes [0], logicMap, 2);

    System.out.println ("Transfering the second");
    nodes [1] = srcManager.transfer (dstManager, nodes [1], logicMap, 2);

    // -- size of mapToTF is #of vars * 2 * 2
    // -- *2 since we use two boolean vars to represent a 4-val var
    // -- *2 since we need primed vars
    int[] mapToTF = new int[ordering.size () << 2];

    for (int i = 0; i < mapToTF.length; i+=2)
      {
	mapToTF [i] = i + 1;
	mapToTF [i+1] = i + 1;
      }
    
    System.out.println ("Renaming things...");
    nodes [1] = dstManager.renameVars (nodes [1], mapToTF);
    
    System.out.println ("Creating result");
    MvSet result = ((JBDDMvSetFactory)dstFactory).createMvSet (nodes);
    
    srcManager = null;
    ddManager = null;

    
    liftEFTT = null;
    liftTTEF = null;
    
    System.out.println ("Running garbage collection");
    System.gc ();
    
    System.out.println ("Done with merging");
    return result;
  }

  
  MvSet mergeMvSets2 (MvSet _set1, MvSet _set2)
  {
    CUBDDMvSet set1 = ((CUBDDMvSet)_set1);
    CUBDDMvSet set2 = ((CUBDDMvSet)_set2);
    
    Cudd cudd = ((CUBDDMvSetFactory)dstFactory).getCudd ();
    int[] ptrs = new int [2];
    ptrs [0] = set1.getBddPtrs ()[0];
    ptrs [1] = set2.getBddPtrs ()[0];

    cudd.ref (ptrs[0]);
    cudd.ref (ptrs[1]);
    return ((CUBDDMvSetFactory)dstFactory).createMvSet (ptrs);
  }

  MvSet mergeMvSetsJoin (MvSet _set1, MvSet _set2)
  {
    System.out.println ("Merging mvSets using Join");
    JMvSet set1 = (JMvSet)_set1;
    JMvSet set2 = (JMvSet)_set2;
    

    
    MDDManager srcManager = ((JMvSetFactory)ddManager).getMddManager ();
    MDDManager dstManager = ((JMvSetFactory)dstFactory).getMddManager ();
    
    int[] logicMap = new int [2];
    
    logicMap [twoVal.getElementIdxByKey ("T")] = 
      fourVal.getElementIdxByKey ("TT");
    logicMap [twoVal.getElementIdxByKey ("F")] =
      fourVal.getElementIdxByKey ("FF");
    

    System.out.println ("Transfering node1");
    MDDNode node1 
      = srcManager.transfer (dstManager, set1.getMddNodes ()[0], logicMap);
    System.out.println ("Transfering node2");
    MDDNode node2
      = srcManager.transfer (dstManager, set2.getMddNodes ()[0], logicMap);
    
//      System.out.println ("Composing");
//      MDDNode compose =
//        dstManager.Apply (node1, node2, FiniteLattice.COMPOSE);
    MDDNode[] mdds = new MDDNode[2];
    mdds [1] = node1;
    mdds [0] = node2;
    


    // -- clean up
    ddManager = null;
    srcManager = null;
    liftEFTT = null;
    liftTTEF = null;
    System.out.println ("Garbage collecting");
    System.gc ();
    
    System.out.println ("Building an MvSet");
    return ((JMvSetFactory)dstFactory).createMvSet (mdds);
    
    
  }

  
  
  MvSet mergeMvSetsMDD (MvSet _set1, MvSet _set2)
  {
    System.out.println ("Merging mvSets using MDD");
    MDDMvSet set1 = (MDDMvSet)_set1;
    MDDMvSet set2 = (MDDMvSet)_set2;
    

    
    MDDManager srcManager = ((MDDMvSetFactory)ddManager).getMddManager ();
    MDDManager dstManager = ((MDDMvSetFactory)dstFactory).getMddManager ();
    
    int[] logicMap = new int [2];
    
    logicMap [twoVal.getElementIdxByKey ("T")] = 
      fourVal.getElementIdxByKey ("TT");
    logicMap [twoVal.getElementIdxByKey ("F")] =
      fourVal.getElementIdxByKey ("FF");

//      logicMap [twoVal.getElementIdxByKey ("T")] = 
//        threeVal.getElementIdxByKey ("T");
//      logicMap [twoVal.getElementIdxByKey ("F")] =
//        threeVal.getElementIdxByKey ("F");

    System.out.println ("Transfering node1");
    MDDNode node1 
      = srcManager.transfer (dstManager, set1.getMddNode (), logicMap);
    System.out.println ("Transfering node2");
    MDDNode node2
      = srcManager.transfer (dstManager, set2.getMddNode (), logicMap);
    

    System.out.println ("Composing");
    MDDNode compose =
      dstManager.Apply (node1, node2, FiniteLattice.COMPOSE);
    
    if (2 != 2)
      compose = node2;


    // -- clean up
    ddManager = null;
    srcManager = null;
    liftEFTT = null;
    liftTTEF = null;
    System.out.println ("Garbage collecting");
    System.gc ();


    System.out.println ("Building an MvSet");
    return ((MDDMvSetFactory)dstFactory).createMvSet (compose);
    
    
  }
  
  MvSet mergeMvSets_cudd (CUBDDMvSet set1, CUBDDMvSet set2)
  {
    System.out.println ("Merging mvsets");
    Cudd srcCudd = ((CUBDDMvSetFactory)ddManager).getCudd ();
    Cudd dstCudd = ((CUBDDMvSetFactory)dstFactory).getCudd ();
    
    
    int[] ptrs = new int [2];

    ptrs [0] = set1.getBddPtrs ()[0];
    ptrs [1] = set2.getBddPtrs ()[0];

    
    System.out.println ("Before the transfer the pointers are different: " +
			(ptrs [0] == ptrs [1] ? "false" : "true"));

    
    System.out.println ("Transfering the first");
    ptrs[0] = srcCudd.transferPtr (dstCudd, ptrs[0], 2);
    System.out.println ("Transfering the second");
    ptrs[1] = srcCudd.transferPtr (dstCudd, ptrs[1], 2);

    // -- size of mapToTF is #of vars * 2 * 2
    // -- *2 since we use two boolean vars to represent a 4-val var
    // -- *2 since we need primed vars
    int[] mapToTF = new int[ordering.size () << 2];

    for (int i = 0; i < mapToTF.length; i+=2)
      {
	mapToTF [i] = i + 1;
	mapToTF [i+1] = i + 1;
      }
    

    int temp = dstCudd.bddPermutePtr (ptrs[1], mapToTF);
    dstCudd.recursiveDeref (ptrs [1]);
    ptrs [1] = temp;

    System.out.println ("After the transfer the pointers are different: " +
			(ptrs [0] == ptrs [1] ? "false" : "true"));
    
    
    MvSet result = ((CUBDDMvSetFactory)dstFactory).createMvSet (ptrs);
    
    srcCudd = null;
    ddManager = null;

    
    liftEFTT = null;
    liftTTEF = null;
    
    System.out.println ("Running garbage collection");
    System.gc ();
    
    System.out.println ("Done with merging");
    return result;
  }
  

  public MvState initializeInitState2 (MvState initState)
  {
    return liftEFTT.initializeInitState (initState);
  }
  
  public MvState initializeInitState (MvState initState)
  {
    boolInitState = new MvState (ordering.size (), twoVal);
    liftEFTT.initializeInitState (boolInitState);

    //return transferToThreeVal (boolInitState, initState);
    return transferToFourVal (boolInitState, initState);
  }
  
  MvState transferToThreeVal (MvState boolState, MvState threeState)
  {
    int t = twoVal.getElementIdxByKey ("T");
    int top = threeVal.getElementIdxByKey ("T");
    int f = twoVal.getElementIdxByKey ("F");
    int bot = threeVal.getElementIdxByKey ("F");

    int[] r = boolState.getRestrictions ();
    
    for (int i = 0; i < r.length; i ++)
      {
	if (r [i] == t)
	  threeState.setValue (i, top);
	else if (r [i] == f)
	  threeState.setValue (i, bot);
      }
    return threeState;
    
  }
  
  MvState transferToFourVal (MvState boolState, MvState fourState)
  {
    int t = twoVal.getElementIdxByKey ("T");
    int f = twoVal.getElementIdxByKey ("F");
    int tt = fourVal.getElementIdxByKey ("TT");
    int ff = fourVal.getElementIdxByKey ("FF");
    
    int[] r = boolState.getRestrictions ();
    
    for (int i = 0; i < r.length; i ++)
      {
	if (r [i] == t)
	  fourState.setValue (i, tt);
	else if (r [i] == f)
	  fourState.setValue (i, ff);
      }
    return fourState;
  }
  

  public static SMVModule initModule (String[] args)
  {
    int floors;
    
    try {
      floors = Integer.parseInt (args [0]);
    } catch (Exception ex) {
      System.out.println ("Hmm, no floor: " + ex);
      floors = 3;
    }
    
    return new MainLiftRCombined (floors);
    
  }
  
  
}


