package edu.toronto.cs.lift;

import edu.toronto.cs.smv.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.algebra.*;

import edu.toronto.cs.util.*;

import java.util.Arrays;

public class MainLiftRTTEF extends MainLiftREFTT
{
  
  public MainLiftRTTEF (String name, int floors)
  {
    super (name);

    lift = new LiftRTTEF ("lift", floors, this);
    landingBtns = new ButtonR [floors];

    for (int i = 0; i < floors; i++)
      landingBtns [i] = new ButtonR ("landingBut" + (i+1));
    
    addSubModules (Arrays.asList (landingBtns));
    addSubModule (lift);
  }
  


  // -- Module intialization routine to be used by the model checker
  public static SMVModule initModule (String[] args)
  {
    System.out.println ("Intializing: " + MainLiftRTTEF.class.getName ());
    System.out.println ("Will refer to this module as \'main\'");
    String moduleName = "main";
    System.out.println ("Expecting one argument: numer of floors");

    int floors;
    try {
      floors = Integer.parseInt (args [0]);
    } catch (Exception ex) {
      System.out.println ("Hmm no arguments, got: " + ex);
      System.out.println ("Assuming 3 floors");
      floors = 3;
    }
    
    
    String latticeFile = "play/gxl/boolean.xml";
    //latticeFile = "play/models/phone/4val-logic.xml";
    FiniteLattice lattice = null;
    System.out.println ("Using " + latticeFile + " as our lattice");
    try {
      lattice = LatticeFactory.parse (latticeFile);
    } catch (Exception ex) {
      System.out.println ("Failed to parse: " + latticeFile);
      ex.printStackTrace ();
      Assert.assert ("Failed to parse the lattice", ex);
    }
    
    MainLiftRTTEF main = new MainLiftRTTEF (moduleName, floors);
    main.setLattice (lattice);
    return main;
  }
  
  
}
