package edu.toronto.cs.mdd;

import edu.toronto.cs.algebra.*;


public class MDDMvProductFunction implements MDDMgr.ApplyFunction
{

  // -- base id for this function for caching
  public static final int baseId = 500;

  FiniteLattice lattice;
  int index;
  
  /**
   * Creates a new <code>MDDMvProductFunction</code> instance.
   *
   * @param _lattice A lattice we are working with
   * @param _index Which view point are we merging
   */
  public MDDMvProductFunction (FiniteLattice _lattice, int _index)
  {
    lattice = _lattice;
    index = _index;
  }
  
  // -- id to be used for caching
  public int getId ()
  {
    return baseId + index;
  }
  
  public boolean isSymetric ()
  {
    return false;
  }
  
  public boolean isCacheable ()
  {
    return true;
  }
  
  // -- apply the function to two arguments
  public int binApply (int a, int b)
  {
    /**** 
	  This operation applies on the level of textual representation of
	  lattice values. We assume that each value is of the form
	  'x1x2x3...' where each 'xi' is either 'T' or 'F'
	  the rest is trivial
     ****/
    char[] data = lattice.getElementKeyByIdx (a).toString ().toCharArray ();
    data [index] = b == lattice.idtimesIdx () ? 'T' : 'F';
    return lattice.getElementIdxByKey (new String (data));
  }
  
  
}
