package edu.toronto.cs.modelchecker;

import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.xkripke.XKripke.*;

import java.util.*;

/****
 **** This interface encapsulates the idea that we need to have a consistent
 **** order on propositions from a XKripke model.
 **** Since it is not clear what is required of such an interface at the
 **** moment it will implement the following parts
 **** 1) Given a XKripkeTransition it returns an Iterator that returns the 
 ****    value of each proposition in order 
 **** 2) Given an order number it will return the name of the variable
 **** 3) Given an order it will return if the proposition is pre-image or 
 ****    post-image (i.e. primed) one
 ****/
public interface PropOrderIf
{
  // -- enumerated type for the 'getPropType' method
  public static final int PRE_IMAGE = 1;
  public static final int POST_IMAGE = 2;
  
  Iterator orderTransition (XKripkeTransition transition); 
  String getPropName (int idx);
  boolean isPreImage (int idx);
  boolean isPostImage (int idx);
  int getPropType (int idx);
  // -- given a name of the property returns its index in the ordering
  int getPropOrder (String propName);
  int getNextPropOrder (String propName);
  int[] getPrimedMap ();
  int[] getPrimedProps ();
  int[] getUnPrimedMap ();
  int[] getUnPrimedProps ();
  // -- number of propositional variables (pre and post)
  int size ();
}
