package edu.toronto.cs.mvset;

import edu.toronto.cs.algebra.*;

/***
 *** A factory to produce MvSets
 ***/
public interface MvSetFactory 
{
  /***
   *** Given a value in L creates a function 
   *** f(x_0, x_1, ...) = value
   ***/
  MvSet createConstant (int value);
  
  /***
   *** Given an argument index (0 <= argIdx < n)
   *** constructs a projection function
   *** f(x_0, x_1, ...) = x_argIdx
   ***/
  MvSet createProjection (int argIdx);


  
  /**
   * <code>createCase</code> creates a case relative to argIdx
   * this is somewhat simillar to Ite (if-then-else) in CUDD
   * but extended to more than two children.
   *
   * @param argIdx an <code>int</code> value
   * @param children an array of  <code>MvSet[]</code> whose size 
   *                 must be equal the size of the logic
   *                 such that children [i] corresponds to the i^th child
   *                 of an mv-set we are creating
   * Note that argIdx must be smaller than any variable in children
   * to preserve ordering.
   * @return a <code>MvSet</code> value
   */
  MvSet createCase (int argIdx, MvSet[] children);
  /***
   *** Creates a point function 
   *** f (args) = value
   ***          = 0 otherwise
   ***/
  MvSet createPoint (int[] args, int value); 
  MvSet createPoint (MvState state, int value);
    
    public static int DONT_CARE = -1;
  MvSet var (int arg, int argVal, int value);

  /***
   *** Gets the lattice for this factory.
   ***/
  FiniteLattice getLattice ();
  
}





