package edu.toronto.cs.prettygraph;

import java.util.*;

import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.mvset.MvState;


class DVEdge
{
  String extraStateName;
  StringBuffer extraState;
  StringBuffer edgesb;

  PrettyGraphEdge edge;

  static Logger log = new Logger ("DVEdge");

  static String EDGE_LABEL_SUFFIX = 
    "-EDGE-LABELLING-NODE-AUTOMATICALLY-GENERATED";
  static String EDGE_LABEL_TYPE = "EDGE-LABELLING-NODE";
  static String EDGE_CONTINUE_SUFFIX = 
    "-CONTINUATION-EDGE-AUTOMATICALLY-GENERATED";
  
  String mainEdgeName;
  

  public void setupPrettyGraphEdge (String nameString)
  {
    //extraState = null;
    // DVEdge dvedge = new DVEdge ();
    mainEdgeName = nameString;
    
    edgesb = Dv.item (mainEdgeName, 
		  Dv.edgebody 
		  ( edge.getType (), 
		   Dv.enum (doAttributes ()), 
		   Dv.ref (doReference ()) ));
    //if (extraState!=null)
    //  {
	//setExtraState (extraState);
	//log.debug ("Setting extra state name to: "+extraStateName);
	//setExtraStateName (extraStateName);
    //      }
    // edgesToDV.put (edge, dvedge);
  }


  public DVEdge (PrettyGraphEdge _edge, NamingScheme namingscheme, String snl)
  {
    this (_edge, snl + namingscheme.getName (_edge));
  }
  

  public DVEdge (PrettyGraphEdge _edge, String _nameString)
  {
    edge = _edge;
    setupPrettyGraphEdge (_nameString);
  }

  public DVEdge (PrettyGraphEdge _edge)
  {
    edge = _edge;
    setupPrettyGraphEdge (edge.getName ());
  }
  
  //public void setEdgeSB (StringBuffer sb)
  //{
  // edge = sb;
  //}
  
  public StringBuffer getEdge ()
  {
    return edgesb;
  }
  
  //public void setExtraState (StringBuffer sb)
  //{
  //  extraState = sb;
  //}
  
  public StringBuffer getExtraState ()
  {
    return extraState;
  }
  
  public void setExtraStateName (String s)
  {
    extraStateName = s;
  }
  
  public String getExtraStateName ()
  {
    return extraStateName;
  }

  protected String doReference ()
  {
    // XXX !! Note that this is, implicitly, the reference for the FIRST edge... (potentially to an intermediate labelling node.
    if (extraState != null)
      return extraStateName;
    
    return ""+((PrettyGraphNode)edge.getDestination ()).getName ();
    
  }

  protected Collection doAttributes ( )
  {
    
    Set resulta = new HashSet ();
    Set resultb;
    doColour (resulta);
    doLineType (resulta);
    if (edge.getLabel () != "")
      {
	log.debug ("Labelled edge: "+edge.getLabel ());
	// Split; everything after here operates on both.
	resultb = (HashSet)((HashSet)resulta).clone ();
	doArrowTips (resulta, resultb);
	doLabel (resulta, resultb); // MUST BE LAST
	log.debug ("Attr-a: "+resulta);
      }
    else
      {
	doArrowTips (resulta);
      }
    return resulta;
  }

    
  protected void doLabel (Set resulta, Set resultb)
  {
    if (edge.getLabel () != "")
      {
	// Bother. Now we have to do something.
	extraStateName =  mainEdgeName +EDGE_LABEL_SUFFIX;
	extraState = Dv.item (extraStateName, 
			      Dv.nodebody 
			      (EDGE_LABEL_TYPE,
			       Dv.enum (labelAttr(edge.getLabel ())), 
			       Dv.enum (continueEdge(resultb, mainEdgeName))));
      }
  }  
   
  protected Set labelAttr (String label)
  {
    Set result = new HashSet ();
    result.add (Dv.attrib ("OBJECT", label));
  //result.add (Dv.attrib ("COLOR", Dv.translateColour (edge.getColour ())));
    result.add (Dv.attrib ("BORDER", "none"));
    for (Iterator it = doTextStyle ().iterator (); it.hasNext ();)
      {
	result.add (it.next ());
      }
    return result;
  }
  
  protected Set continueEdge 
    (Set attr, String label)
  {
    Set result = new HashSet ();
    result.add( Dv.item (mainEdgeName+EDGE_CONTINUE_SUFFIX, 
		  Dv.edgebody 
		  ( edge.getType (), 
		   Dv.enum (attr), 
		    Dv.ref (""+((PrettyGraphNode)edge.getDestination ())
			    .getName ()) )) );
    return result;
  }

  protected Set doColour (Set result)
  {
    if (edge.getColour () != "")
      result.add (Dv.attrib ("EDGECOLOR", 
			     Dv.translateColour (edge.getColour ())));
    return result;
  }
  
  protected Set doTextStyle ()
  {
    Set result = new HashSet ();
    if (edge.getFontFamily ()!= "")
      result.add (Dv.attrib("FONTFAMILY", edge.getFontFamily ()));
    if (edge.getFontStyle ()!= "")
      result.add (Dv.attrib("FONTSTYLE", edge.getFontStyle ()));
    return result;
  }
  
  protected Set doLineType (Set result)
  {
    String attribute = "EDGEPATTERN";
    if (edge.getLineNum () != 0)
      {
	if (edge.getLineNum () == PrettyConstants.SINGLE_LINE)
	  {
	    result.add (Dv.attrib (attribute, "single"));
	    return result;
	  }
	else if (edge.getLineNum () == PrettyConstants.DOUBLE_LINE)
	  {
	    result.add (Dv.attrib (attribute, "double"));
	    return result;
	  }
      }
    else if (edge.getLineThickness () != 0)
      {
	if (edge.getLineThickness ()== PrettyConstants.THIN_LINE)
	  {
	    result.add (Dv.attrib (attribute, "single;solid;1;0"));
	    return result;
	  }
	else if (edge.getLineThickness ()== PrettyConstants.THICK_LINE)
	  {
	    result.add (Dv.attrib (attribute, "thick"));
	    return result;
	  }
      }
    // XXX Should implement dotted line here.
    else if (edge.getLineSolidity () != 0)
      {
	if (edge.getLineSolidity () == PrettyConstants.NO_LINE)
	  {
	    result.add (Dv.attrib (attribute, ";;0;"));
	    return result;
	  }
	// So far as I know, dv doesn't support dotted borders...
      }
    //else
    //  {
    //	result.add (Dv.attrib (attribute, "single;solid;1;0"));
    // }
    return result;
  }

  protected void doArrowTips (Set result)
  {
    // Right. First off, _DIR: none, double, inverse normal
    if (edge.getArrowUp ()!=0)
      {
	if (edge.getArrowDown ()!=0)
	  {
	    result.add (Dv.attrib("_DIR", "double"));
	    result.add (Dv.attrib("HEAD", Dv.ARROWS[edge.getArrowDown ()])); // Note default.
	  }
	else
	  {
	    result.add (Dv.attrib("_DIR", "inverse"));
	    result.add (Dv.attrib("HEAD", Dv.ARROWS[edge.getArrowUp ()]));
	  }
      }
    else
      {
	if (edge.getArrowDown ()!=0)
	  {
	    result.add (Dv.attrib("_DIR", "normal"));
	    result.add (Dv.attrib("HEAD", Dv.ARROWS[edge.getArrowDown ()])); // Note default.
	  }
	else
	  {
	    result.add (Dv.attrib("_DIR", "none"));
	  }
      }
  }
  
  protected void doArrowTips (Set resulta, Set resultb)
  {
    // Right. First off, _DIR: none, double, inverse normal
    if (edge.getArrowUp ()!=0)
      {
	resulta.add (Dv.attrib ("_DIR", "inverse"));
	resulta.add (Dv.attrib ("HEAD", Dv.ARROWS[edge.getArrowUp ()]));
      }
    else 
      {
	resulta.add (Dv.attrib ("_DIR", "none"));
      }
    if (edge.getArrowDown ()!=0)
      {
	resultb.add (Dv.attrib ("_DIR", "normal"));
	resultb.add (Dv.attrib ("HEAD", Dv.ARROWS[edge.getArrowDown ()]));
      }
    else 
      {
	resultb.add (Dv.attrib ("_DIR", "none"));
      }
  }
  


}







