package edu.toronto.cs.prettygraph;

import java.util.*;

import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.algebra.*;


public class PrettyGraphMDDVisitor
{
  static Logger log = new Logger ("PrettyGraphMDDVisitor");

  DGraph pretty;
  PrettyMDDNodeFactory factory;
  Map nodeMap;
  Set visited;

  public static DGraph getPrettyRep (MDDNode node, PropOrderIf prop, Map colourmap)
  {
    PrettyGraphMDDVisitor v = new PrettyGraphMDDVisitor ();
    DGraph temp = v.generate (node);
    log.debug ("About to colour.");
    Algebra lat = node.getManager ().getAlgebra ();
    PrettyColouringStrategy colourer = 
      new PrettyMDDColouringStrategy (prop, colourmap, lat,
				      new PrettyMDDEdgeColouring 
					(prop, colourmap, lat));
    for (Iterator it = DGDepthTraverse.getIterator (temp); 
	 it.hasNext ();)
      {
	colourer.visit ((DGNode)it.next (), null);
      }
    return temp;
  }

  public static DGraph getPrettyRep (MDDNode node, PropOrderIf prop)
  {
    return getPrettyRep (node, prop, PrettyConstants.singletonMap);
  }
  
  public static DGraph getPrettyRep (MDDNode node)
  {
    PrettyGraphMDDVisitor v = new PrettyGraphMDDVisitor ();
    DGraph temp = v.generate (node);
    log.debug ("No colouring possible.");
    return temp;
  }

  public PrettyGraphMDDVisitor ()
  {
    pretty = new DGraph (new PrettyMDDNodeFactory ());
    factory = (PrettyMDDNodeFactory)pretty.getFactory ();
    nodeMap = new HashMap ();
    visited = new HashSet ();
  }
  

  public DGraph generate (MDDNode node)
  {
    recurseThrough (node);
    PrettyGraphMDDNode tnode = (PrettyGraphMDDNode) pretty.getRootNodes ()
			       .iterator ().next ();
    //    log.debug ("edges: "+((DGMultiEdge)tnode.getEdge (((DGNode)tnode.getSuccessors ().iterator (). next ()))).getValues ());
    return pretty;
  }
  
  private void recurseThrough (MDDNode node)
  {
    if (visited.contains (node))
      return;
    log.debug ("Recursing through "+node);
    visit (node);
    visited.add (node);
    for (int i=0; i<node.howManyChildren (); i++)
      recurseThrough (node.getChild (i));
  }
  
  protected void doInitial (MDDNode node)
  {
    log.debug ("Doing initial node: "+node);
    PrettyGraphNode newnode = ((PrettyGraphMDDNode)
      factory.createNode (""+getUniqueID ()))
      .setMDD (node);
    pretty.addRootNode (newnode);
    nodeMap.put (node, newnode);
  }
  
  public void visit (MDDNode node)
  {
    if (nodeMap.get (node) == null)
      doInitial (node);

    PrettyGraphMDDNode pnode = (PrettyGraphMDDNode) nodeMap.get (node);
    // now, cycle through the children (ordered!)
    //if (!node.isConstant ())
    for (int i=0; i<node.howManyChildren (); i++)
      {
	log.debug (i + " out of "+node.howManyChildren ());
	PrettyGraphMDDNode nnode;
	nnode = obtainNode (pnode, node, i);
	pnode.add (nnode, i);
      }
  }

  static int n = 0;
  public static String getUniqueID ()
  {
    return (n++) + ":";
  }
  
  
  public PrettyGraphMDDNode obtainNode (PrettyGraphMDDNode pnode, 
					MDDNode node, int i)
  {
    if (nodeMap.get (node.getChild (i)) != null)
      return (PrettyGraphMDDNode) nodeMap.get (node.getChild (i));
    
    PrettyGraphMDDNode nnode;

    if (!node.getChild (i).isConstant ())
      nnode = ((PrettyGraphMDDNode)
	       factory.createNode (""+getUniqueID ()))
	       //factory.createNode (""+pnode.getName ()+"-"+i+"-"
	       //    +node.getChild (i)))
	.setMDD (node.getChild (i));
    else
      nnode = ((PrettyGraphMDDNode)
	       factory.createNode (""+node.getChild (i)))
	.setMDD (node.getChild (i));

    nodeMap.put (node.getChild (i), nnode);

    log.debug ("Created node "+nnode.getName ());
    return nnode;
  }

  
}


