package edu.toronto.cs.prettygraph;

import java.util.*;

import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.mvset.MvState;


public class PrettyGraphXKVisitor extends XKripke.XKripkeVisitor
{
  static Logger log = new Logger ("PrettyGraphXKVisitor");
  

  DGraph dv;
  PrettyXKNodeFactory ourfactory;
  //Map allnodes;

  public static DGraph getPrettyXKGraph (XKripke xk)
  {
    return getPrettyXKGraph (xk, PrettyConstants.singletonMap);
  }

  public static DGraph getPrettyXKGraph (XKripke xk, Map colourMap)
  {
    PrettyGraphXKVisitor pv = new PrettyGraphXKVisitor ();
    /* xktraverser = new XKTreeTraverser (xk);
       XKTraversals.traverseWith (pv); */
    int i=0;
    for (Iterator it = XKDepthTraverse.getIterator (xk); it.hasNext (); i++)
      {
	pv.visitXKripke ((XKripke.XKripkeState)it.next ());
      }
    log.debug ("Finished building graph in "+i+" steps.");
    DGraph result = pv.getGraph ();
    PrettyColouringStrategy colourer = 
      new PrettyXKColourer (new PrettyGraphXKEdger (colourMap));
    for (Iterator it = DGDepthTraverse.getIterator (result); it.hasNext ();)
      {
	colourer.visit ((DGNode)it.next (), null);
      }
    return result;
  }

  public DGraph getGraph ()
  {
    return dv;
  }
  
  public PrettyGraphXKVisitor ()
  {
    dv = new DGraph (new PrettyXKNodeFactory ());
    ourfactory = (PrettyXKNodeFactory) dv.getFactory ();  
  }

  public void visitXKripke (XKripke.XKripkeState state)
  {
    Set transitions = state.getXKripke ().getTransitions ();
    for (Iterator it = transitions.iterator (); it.hasNext ();)
      {
	XKripke.XKripkeTransition transition = 
	  (XKripke.XKripkeTransition) it.next ();

	// XXX Not only is this inefficient, I'm doing it twice (also
	// in XKDepthTraverse)
	if (transition.getSrc () != state)
	  continue;
	
	XKripke.XKripkeState newstate = transition.getDst ();
	log.debug ("retrieving node "+state.getName ());

	// Creates it, or returns it if it already exists.
	PrettyGraphXKNode oldnode = (PrettyGraphXKNode) 
	  ourfactory.createNode (null, state);
	if (state.isInitial ())
	  dv.addRootNode (oldnode);
	PrettyGraphXKNode newnode = (PrettyGraphXKNode) 
	  ((PrettyXKNodeFactory)ourfactory)
	  .createNode ((DGUniqueNode)oldnode, (Object)newstate);
	oldnode.add (newnode, 
		     new PrettyXKTransition (oldnode, newnode, transition));
	log.debug ("Just created nodes for and added transition from "
		   +StringUtil.doEscapes (""+state.getName ())+" to "
		   +StringUtil.doEscapes (""+newstate.getName ())+"...");
	
      }
  }
}




