package edu.toronto.cs.prettygraph;

import java.util.*;

import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.keg.*;
import edu.toronto.cs.mvset.MvState;
import edu.toronto.cs.algebra.*;

public class PrettyKegEdgeColouring extends PrettyGraphEdgeVisitorAncestor
{

  Map colorMap;

  static Logger log = new Logger ("PrettyKegEdgeColouring");

  public static Object STS_COLOUR = java.awt.Color.gray;
  public static int STS_LINETHICK = 1;
  public static int STS_LINENUM = PrettyConstants.DOUBLE_LINE;
  public static int STS_HEAD = PrettyConstants.POINTY_ARROW;
  public static boolean LABEL_STS = true;

  public static Object CC_COLOUR = new java.awt.Color (70, 130, 180);
  public static int CC_LINETHICK = PrettyConstants.THIN_LINE;
  public static int CC_LINENUM = PrettyConstants.SINGLE_LINE;
  public static int CC_HEAD = PrettyConstants.POINTY_FILLED_ARROW;
  
  public PrettyKegEdgeColouring (Map colourMap)
  {
    colorMap = colourMap;
  }
  
  public void visitPrettyGraphEdge (PrettyGraphEdge edge, Object stateinfo)
  {
    colourCtlToCtl (edge);
  }

  public void visitPrettyKegEdge (PrettyKegEdge edge, Object stateinfo)
  {
    log.debug ("    and it's a DVKegEdge");
    // right, what are we dealing with here? ctl -> state or state->state
    if (((PrettyGraphNode) edge.getOrigin ()).getType () 
	== PrettyConstants.ST_TYPE)
      //state->state
      colourStateToState (edge);
    else if 
      (((PrettyGraphNode) edge.getOrigin ()).getType () 
       == PrettyConstants.CTL_TYPE)
      colourCtlToState (edge);
  }
  
  public void colourCtlToCtl (PrettyGraphEdge edge)
  {
    edge.setColour (CC_COLOUR)
      .setLineThickness (CC_LINETHICK)
      .setArrowUp (CC_HEAD);
  }

  private void colourStateToState (PrettyKegEdge edge)
  {
    int trans = ((KEGStateTransition)edge.getKegEdge ()).getValue ();
    Algebra lattice = 
      ((KEGStateNode)((KEGStateTransition)edge.getKegEdge ()).getOrigin ())
      .getState ().getLattice ();
    Object val = lattice.getElementKeyByIdx (trans);
    // XXX Change to allow for case colorMap.get(val)=null
    if (colorMap.get (val) != null)
      edge.setColour (colorMap.get (val));
    else
      edge.setColour (STS_COLOUR);

    edge.setLineNum (STS_LINENUM)
      .setLineThickness (STS_LINETHICK)
      .setArrowDown (STS_HEAD);

    if (LABEL_STS)
      edge.setLabel (val.toString ()); 
  }
  
  private void colourCtlToState (PrettyKegEdge edge)
  {
    colourCtlToCtl (edge);
    PrettyKegGraphNode origin = (PrettyKegGraphNode) 
      ((PrettyKegGraphNode)edge.getOrigin ()).getOwner ();
    
    PrettyKegGraphNode dest = (PrettyKegGraphNode) edge.getDestination ();
    
    KEGStateNode kegorigin = (KEGStateNode)origin.getKegNode ();

    KEGStateNode kegdest = (KEGStateNode)dest.getKegNode ();

    Assert.assert (!(kegdest==null && kegorigin==null), "both null");
    Assert.assert (kegdest!=null, "kd null");
    Assert.assert (kegorigin!=null, "ko null");
    KEGStateTransition trans = 
      (KEGStateTransition)kegorigin.getEdge (kegdest);

    Algebra lattice = kegorigin.getState ().getLattice ();
    Object val = lattice.getElementKeyByIdx (trans.getValue ());

    if (colorMap.get (val) != null)
      edge.setColour (colorMap.get (val));
    else
      edge.setColour (PrettyConstants.defaultColour);
    

  }
}









