package edu.toronto.cs.smv;

import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.tlq.*;
import edu.toronto.cs.modelchecker.*;

import edu.toronto.cs.mvset.MDDMvSet.*;


import java.util.*;
import java.io.*;


public class SMVBddModuleMerge extends SMVModule
{

  SMVBddModule module1;
  SMVBddModule module2;

  FiniteLattice twoVal;
  FiniteLattice threeVal;
  FiniteLattice fourVal;

  MvState boolInitState;

  MvSetFactory dstFactory;

  public SMVBddModuleMerge (SMVBddModule _module1, SMVBddModule _module2)
  {
    super ("");
    try {
      twoVal = LatticeFactory.parse ("play/gxl/boolean.xml");
      fourVal = LatticeFactory.parse ("play/models/phone/4val-logic.xml");
    } catch (Exception ex) {
      Assert.assert ("Hmm", ex);
    }
    
    module1 = _module1;
    module2 = _module2;
    setLattice (twoVal);
  }

  public void setMvSetFactory (MvSetFactory factory)
  {
    dstFactory = factory;
    MvSetFactory boolFactory = null;
    boolFactory = MDDMvSet.newMvSetFactory (twoVal, ordering.size ());
    super.setMvSetFactory (boolFactory); // ?
    module1.setMvSetFactory (boolFactory);
    module2.setMvSetFactory (boolFactory);
  }

  public void setLattice (FiniteLattice lattice)
  {
    module1.setLattice (lattice);
    module2.setLattice (lattice);
  }
  
  public FiniteLattice getLattice ()
  {
    return fourVal;
  }

  public String[] getVars ()
  {
    // -- this is needed for initialization
    module1.getVars ();
    return module2.getVars ();
  }
  public void setVarOrdering (PropOrderIf ordering)
  {
    this.ordering = ordering;    
    System.out.println ("Calling setVarOrdering on sub modules");
    module1.setVarOrdering (ordering);
    module2.setVarOrdering (ordering);
  }

  public MvSet computeTrans ()
  {
    return mergeMvSets (module1.computeTrans (), module2.computeTrans ());
  }

  MvSet mergeMvSets (MvSet set1, MvSet set2)
  {
    if (set1 instanceof MDDMvSet)
      return mergeMvSetsMDD (set1, set2);
    else
      Assert.assert ("Not implemented");
    return null;
  }
  
  MvSet mergeMvSetsMDD (MvSet _set1, MvSet _set2)
  {
    System.out.println ("Merging mvSets using MDD");
    MDDMvSet set1 = (MDDMvSet)_set1;
    MDDMvSet set2 = (MDDMvSet)_set2;
    

    
    MDDManager srcManager = ((MDDMvSetFactory)ddManager).getMddManager ();
    MDDManager dstManager = ((MDDMvSetFactory)dstFactory).getMddManager ();
    
    int[] logicMap = new int [2];
    
    logicMap [twoVal.getElementIdxByKey ("T")] = 
      fourVal.getElementIdxByKey ("TT");
    logicMap [twoVal.getElementIdxByKey ("F")] =
      fourVal.getElementIdxByKey ("FF");


    System.out.println ("Transfering node1");
    MDDNode node1 
      = srcManager.transfer (dstManager, set1.getMddNode (), logicMap);
    System.out.println ("Transfering node2");
    MDDNode node2
      = srcManager.transfer (dstManager, set2.getMddNode (), logicMap);
    

    System.out.println ("Composing");
    MDDNode compose =
      dstManager.Apply (node1, node2, FiniteLattice.COMPOSE);

    // -- clean up
    ddManager = null;
    srcManager = null;
    module1 = null;
    module2 = null;
    System.out.println ("Garbage collecting");
    System.gc ();


    System.out.println ("Building an MvSet");
    return ((MDDMvSetFactory)dstFactory).createMvSet (compose);
    
    
  }
  

  public EnumeratedTypes getEnumTypes() 
  {
    // -- forward to one of the BddModules we have
    return module1.getEnumTypes ();
  }  

  public MvState initializeInitState (MvState initState)
  {
    boolInitState = new MvState (ordering.size (), twoVal);
    return transferToFourVal (module1.initializeInitState (boolInitState),
			      initState);
    
  }
  
  MvState transferToFourVal (MvState boolState, MvState fourState)
  {
    int t = twoVal.getElementIdxByKey ("T");
    int f = twoVal.getElementIdxByKey ("F");
    int tt = fourVal.getElementIdxByKey ("TT");
    int ff = fourVal.getElementIdxByKey ("FF");
    
    int[] r = boolState.getRestrictions ();
    
    for (int i = 0; i < r.length; i ++)
      {
	if (r [i] == t)
	  fourState.setValue (i, tt);
	else if (r [i] == f)
	  fourState.setValue (i, ff);
      }
    return fourState;
  }

  
  public static SMVModule initModule (String[] args)
  {
    /***
	args [0] -- prop ordering file
	args [1] -- bdd file
	args [2] -- bdd file 2
	args [3] -- initial state file
	args [4] -- enumerted types file

    ***/
    //    String latticeName = args [0];
    
    String propOrderFileName = args [0];
    String bddFileName = args [1];
    String bddFileName2 = args [2];
    String initFileName = args[3];
    String etypesFileName = args [4];
        
    String[] vars = SMVBddModule.computeVars (propOrderFileName);

    
    SMVBddModule module1 = new SMVBddModule (vars, bddFileName,
					     initFileName, etypesFileName);
    SMVBddModule module2 = new SMVBddModule (vars, bddFileName2, 
					     initFileName, etypesFileName);
    
    
    
    
    return new SMVBddModuleMerge (module1, module2);
  }
  


  
}
