package edu.toronto.cs.smv;

import edu.toronto.cs.ctl.*;
import java.util.*;

public class SMVEnumTypeExpander extends MacroExpander
{
  // -- the enumerated types we know about
  EnumeratedTypes enumTypes;
  

  public SMVEnumTypeExpander (EnumeratedTypes _enumTypes)
  {
    enumTypes = _enumTypes;
  }
  

  public void reset ()
  {
    // -- no reset required
  }
  
  public CTLNode expand (CTLNode ctl)
  {
    // -- we only handle equality and PlaceHolder nodes
    if (ctl instanceof CTLEqualsNode)
      return expandEq ((CTLEqualsNode) ctl);
//     else if (ctl instanceof CTLPlaceholderNode)
//       return expandPlaceHolder ((CTLPlaceholderNode) ctl);
    else
      return ctl;
  }

  // -- this was moved into PlaceholderManager
  CTLNode expandPlaceHolder (CTLPlaceholderNode ctl)
  {

    // -- get the old propositions
    String[] oldProps = (String[]) ctl.getProps ();
    
    // -- a list of new propositions
    List newProps = new ArrayList ();
    
    // -- expand every oldProp which is of an enumerated type into 
    // -- a set of boolean variables
    for (int i = 0; i < oldProps.length; i++)
      {
	if (enumTypes.isEnumeratedVariable (oldProps [i]))
	  for (int j = 0; j < enumTypes.getRangeBits (oldProps [i]); j++)
	    newProps.add (oldProps [i] + j);
	else
	  newProps.add (oldProps [i]);
      }
    
    // -- see if we did anything
    if (newProps.size () == oldProps.length) return ctl;
    
    // -- some expansion took place
    return CTLFactory.createCTLPlaceholderNode 
      (ctl.getName (), newProps.toArray (new String [newProps.size ()]));
    
  }
  
  // -- end of public interface
  CTLNode expandEq (CTLEqualsNode ctl)
  {
    // -- both left and right nodes have to be atom nodes -- and not subclasses
    if (ctl.getRight ().getClass () != CTLAtomNode.class ||
	ctl.getLeft ().getClass () != CTLAtomNode.class)
      return ctl;
    
    CTLAtomNode left = (CTLAtomNode)ctl.getLeft ();
    CTLAtomNode right = (CTLAtomNode)ctl.getRight ();
    
    // -- one should be an enumerated type value, the other 
    // -- and enumerated type variable
    if (enumTypes.isEnumeratedVariable (left.getName ().toString ()) &&
	enumTypes.variableHasValue (left.getName ().toString (), 
				    right.getName ().toString ()))
      return buildEq (left.getName ().toString (), 
		      right.getName ().toString ());
    else if (enumTypes.isEnumeratedVariable (right.getName ().toString ()) &&
	     enumTypes.variableHasValue (right.getName ().toString (), 
					 left.getName ().toString ()))
      return buildEq (right.getName ().toString (), 
		      left.getName ().toString ());
    else
      // -- something is wrong so we don't handle it
      return ctl;
  }
  

  // -- given an enumerated type variable and its value
  // -- expands varName = varVal 
  // -- CTL node into appropriate expression in varName_{idx}
  CTLNode buildEq (String varName, String varVal)
  {
    CTLNode result = null;
    
    int[] encodings = enumTypes.getAllEncodings (varName, varVal);
    
    for (int i = 0; i < encodings.length; i++)
      {
	if (result == null)
	  result = buildSingleEncoding (varName, encodings [i]);
	else
	  result = result.or (buildSingleEncoding (varName, encodings [i]));
      }
//     System.out.println ("Expanding: " + varName + " = " + varVal);
//     System.out.println ("Result: ");
//     System.out.println (result);
    return result;
  }

  CTLNode buildSingleEncoding (String varName, int encoding)
  {
    // -- find out how many bits are important
    int rangeBits = enumTypes.getRangeBits (varName);

    // -- place to store our result
    CTLNode result = null;
    
    for (int i = 0; i < rangeBits; i++)
      {
	// -- create CTL var node
	CTLNode curVar = CTLFactory.createCTLAtomNode (varName + i);

	// -- check if this bit is positive or not
	if ((encoding & 1) == 0)
	  curVar = curVar.neg ();
	
	// -- done with that bit
	encoding = encoding >>> 1;
	
	// -- add to our result set
	if (result == null) result = curVar;
	else
	  result = result.and (curVar);
	
      }
    return result;
  }
  
  
}
