package edu.toronto.cs.util.dgraph;

import java.util.*; // We all love java.util ...

import edu.toronto.cs.util.*;

/** A basic directed graph.
@author CDTW */
public class DGNode implements DGNodeVisitable
{
  static Logger log = new Logger ("DGNode");
  
  protected Set successors;
  public Object data;
  public Map edgeData;
  public DGNodeFactory owner;
  public DGraph graph=null;
  //  public boolean rootNode = false;

  protected DGNode (DGNodeFactory _owner)
  {
    owner = _owner;
    successors = new HashSet ();
    edgeData = new HashMap ();
  }

  protected void addSuccessor (DGNode successor)
  {
    if (checkValid(successor))
      {	
	doAddition (successor);
	
      }
  }

  protected void doAddition (DGNode successor)
  {
    successors.add (successor);
    setupEdge (successor);
  }

  //  protected void setRootNode (boolean b)
//    {
//      rootNode = b;
//    }
  
  public boolean isRootNode ()
  {
    return graph.isRootNode (this);
  }
  
  public void accept (DGraphVisitor visitor, Object stateinfo)
  {
    log.debug ("VISIT DGNode by DGV");
    visitor.visitDGNode (this, stateinfo);
  }

  public void addEdge (DGNode successor, DGEdge edge)
  {
    //  Set edges;
//      if (edgeData.get (successor) != null)
//        edges = (Set) edgeData.get (successor);
//      else
//        edges = new HashSet ();
//      edges.add (edge);
    edgeData.put (successor, edge);
  }

  // Factory method (essentially) for first edge
  public void setupEdge (DGNode successor)
  {
    addEdge (successor, new DGEdge (this, successor));
  }

  public DGEdge getEdge (Object successor)
  {
    return (DGEdge)edgeData.get (successor);
  }

  public Set getAllEdges ()
  {
    Set result = new HashSet ();
    for (Iterator it = edgeData.values (). iterator (); it.hasNext ();)
      {
	result.add (it.next ());
      }
    return result;
  }
  

  public boolean add (DGNode successor)
  {
    if (graph==null)
      return false;
    // graph.addSuccessorTo(this, successor);
    if (!graph.isOkToAdd (successor))
      return false;
    if (!owner.addOk (successor))
      return false;
    addSuccessor (successor);
    successor.setGraph (graph);
    return true;
  }
  
  /**
   * Get the value of data.
   * @return value of data.
   */
  public Object getData() 
  {
    return data;
  }
  
  /**
   * Set the value of data.
   * @param v  Value to assign to data.
   */
  public void setData(Object  v) 
  {
    this.data = v;
  }
  

  public Set getSuccessors ()
  {
    return (HashSet)successors; 
  }

  public boolean checkValid (DGNode node)
  {
    if (node==this)
      return false;
    for (Iterator it = successors.iterator (); it.hasNext ();)
      {
	if (node==it.next ())
	  return false;
      }
    return true;
  } 

  public DGraph getGraph ()
  {
    return graph;
  }

  public DGNodeFactory getFactory ()
  {
    return owner;
  }

  public void setFOwner (DGNodeFactory f)
  {
    owner = f;
  }

  public boolean isMember (DGraph _graph)
  {
    return (graph==_graph);
  }

  protected void setGraph (DGraph _graph)
  {
    graph = _graph;
  }
}



    

